@@ -13,27 +13,27 @@ use Spp::Cursor;
 
 sub match_rule {
    my ($rule, $cursor) = @_;
-   my ($name, $value)  = @{$rule};
+   my ($name, $atom)  = @{$rule};
    given ($name) {
-      when ('Rules')    { match_rules($value, $cursor)    }
-      when ('Group')    { match_rules($value, $cursor)    }
-      when ('Branch')   { match_branch($value, $cursor)   }
-      when ('Rept')     { match_rept($value, $cursor)     }
-      when ('Look')     { match_look($value, $cursor)     }
-      when ('Cclass')   { match_cclass($value, $cursor)   }
-      when ('Chclass')  { match_chclass($value, $cursor)  }
-      when ('Nchclass') { match_nchclass($value, $cursor) }
-      when ('Str')      { match_str($value, $cursor)      }
-      when ('Char')     { match_char($value, $cursor)     }
-      when ('Assert')   { match_assert($value, $cursor)   }
-      when ('Not')      { match_not($value, $cursor)      }
-      when ('Till')     { match_till($value, $cursor)     }
-      when ('Rtoken')   { match_rtoken($value, $cursor)   }
-      when ('Ctoken')   { match_ctoken($value, $cursor)   }
-      when ('Ntoken')   { match_ntoken($value, $cursor)   }
-      when ('Any')      { match_any($value, $cursor)      }
-      when ('Expr')     { match_expr($value, $cursor)     }
-      when ('Sym')      { match_sym($value, $cursor)      }
+      when ('Rules')    { match_group($atom, $cursor)   }
+      when ('Group')    { match_group($atom, $cursor)   }
+      when ('Branch')   { match_branch($atom, $cursor)  }
+      when ('Rept')     { match_rept($atom, $cursor)    }
+      when ('Look')     { match_look($atom, $cursor)    }
+      when ('Cclass')   { match_cclass($atom, $cursor)  }
+      when ('Chclass')  { match_chclass($atom, $cursor) }
+      when ('Nchclass') { match_nchclass($atom, $cursor)}
+      when ('Str')      { match_str($atom, $cursor)     }
+      when ('Char')     { match_char($atom, $cursor)    }
+      when ('Assert')   { match_assert($atom, $cursor)  }
+      when ('Not')      { match_not($atom, $cursor)     }
+      when ('Till')     { match_till($atom, $cursor)    }
+      when ('Rtoken')   { match_rtoken($atom, $cursor)  }
+      when ('Ctoken')   { match_ctoken($atom, $cursor)  }
+      when ('Ntoken')   { match_ntoken($atom, $cursor)  }
+      when ('Any')      { match_any($atom, $cursor)     }
+      when ('Expr')     { match_expr($atom, $cursor)    }
+      when ('Sym')      { match_sym($atom, $cursor)     }
       default { 
          error("unknown rule type |$name|!")
       }
@@ -43,8 +43,8 @@ sub match_rule {
 sub match_any {
    my ($any, $cursor) = @_;
    my $char = $cursor->get_char;
-   if ($char eq chr(0)) { return ['false'] }
-   $cursor->go;
+   if ($char eq chr(0)) { return False }
+   $cursor->to_next;
    return $char;
 }
 
@@ -52,33 +52,33 @@ sub match_assert {
    my ($assert, $cursor) = @_;
    given ($assert) {
       when ('^') {
-         return ['true'] if $cursor->off == 0;
-         return ['false'];
+         return True if $cursor->off == 0;
+         return False;
       }
       when ('$') {
-         return ['true'] if $cursor->get_char eq chr(0);
-         return ['false'];
+         return True if $cursor->get_char eq chr(0);
+         return False;
       }
       when ('^^') {
-         return ['true'] if $cursor->pre_char eq "\n";
-         return ['true'] if $cursor->off == 0;
-         return ['false'];
+         return True if $cursor->pre_char eq "\n";
+         return True if $cursor->off == 0;
+         return False;
       }
       when ('$$') {
-         return ['true'] if $cursor->get_char eq "\n";
-         return ['true'] if $cursor->get_char eq chr(0);
-         return ['false'];
+         return True if $cursor->get_char eq "\n";
+         return True if $cursor->get_char eq chr(0);
+         return False;
       }
       default { error("error assert char: <$assert>!") }
    }
 }
 
-sub match_rules {
+sub match_group {
    my ($rules, $cursor) = @_;
-   my $gather = ['true'];
+   my $gather = True;
    for my $rule (@{$rules}) {
       my $match = match_rule($rule, $cursor);
-      return ['false'] if is_false($match);
+      return False if is_false($match);
       $gather = gather_match($gather, $match);
    }
    return $gather;
@@ -87,35 +87,37 @@ sub match_rules {
 sub match_branch {
    my ($branch, $cursor) = @_;
    my $cache     = $cursor->cache;
-   my $max_match = ['false'];
+   my $max_match = False;
    my $max_cache = $cache;
    my $max_off   = -1;
    for my $rule (@{$branch}) {
       my $match = match_rule($rule, $cursor);
-      if (is_match($match)) {
+      if (!is_false($match)) {
          if ($cursor->off > $max_off) {
             $max_off = $cursor->off;
             $max_cache = $cursor->cache;
             $max_match = $match;
          }
       }
-      $cursor->recover($cache);
+      $cursor->reset_cache($cache);
    }
-   $cursor->recover($max_cache);
+   $cursor->reset_cache($max_cache);
    return $max_match;
 }
 
 sub match_ntoken {
    my ($name, $cursor) = @_;
    my $rule  = $cursor->{'ns'}{$name};
+   my $from  = $cursor->off;
    my $cache = $cursor->cache;
    my $match = match_rule($rule, $cursor);
    return $match if is_bool($match);
-   my $from  = $cache->[1];
    my $len   = $cursor->off - $from;
-   my $str   = substr($cursor->str, $from, $len);
-   my $cname = '@' . $name;
-   $cursor->{'ns'}{$cname} = ['Str', $str];
+   if ($len > 0) {
+      my $str   = substr($cursor->str, $from, $len);
+      my $cname = '$' . $name;
+      $cursor->{'ns'}{$cname} = ['Str', $str];
+   }
    push @{$cache}, $len;
    return name_match($name, $match, $cache);
 }
@@ -127,9 +129,11 @@ sub match_ctoken {
    my $match = match_rule($rule, $cursor);
    return $match if is_bool($match);
    my $len   = $cursor->off - $from;
-   my $str   = substr($cursor->str, $from, $len);
-   my $cname = '@' . $name;
-   $cursor->{'ns'}{$cname} = ['Str', $str];
+   if ($len > 0) {
+      my $str   = substr($cursor->str, $from, $len);
+      my $cname = '$' . $name;
+      $cursor->{'ns'}{$cname} = ['Str', $str];
+   }
    return $match;
 }
 
@@ -137,8 +141,8 @@ sub match_rtoken {
    my ($name, $cursor) = @_;
    my $rule = $cursor->{'ns'}{$name};
    my $match = match_rule($rule, $cursor);
-   return ['false'] if is_false($match);
-   return ['true'];
+   return False if is_false($match);
+   return True;
 }
 
 sub match_not {
@@ -146,10 +150,10 @@ sub match_not {
    my $cache = $cursor->cache;
    my $match = match_rule($rule, $cursor);
    if (is_false($match)) {
-      $cursor->recover($cache);
-      return ['true'];
+      $cursor->reset_cache($cache);
+      return True;
    }
-   return ['false'];
+   return False;
 }
 
 sub match_till {
@@ -157,20 +161,22 @@ sub match_till {
    my @buf = ();
    while ($cursor->off < $cursor->len) {
       my $char = $cursor->get_char;
+      my $cache = $cursor->cache;
       my $match = match_rule($rule, $cursor);
-      if (is_match($match)) {
-         my $gather_str - join '', @buf;
+      if (!is_false($match)) {
+         my $gather_str = join '', @buf;
          return gather_match($gather_str, $match);
       }
       push @buf, $char;
-      $cursor->go;
+      $cursor->reset_cache($cache);
+      $cursor->to_next;
    }
-   return ['false'];
+   return False;
 }
 
 sub match_rept {
    my ($rule, $cursor) = @_;
-   my $gather = ['true'];
+   my $gather = True;
    my $time   = 0;
    my ($rept, $atom) = @{$rule};
    my ($min,  $max)  = get_rept_time($rept);
@@ -178,8 +184,8 @@ sub match_rept {
       my $cache = $cursor->cache;
       my $match = match_rule($atom, $cursor);
       if (is_false($match)) {
-         return ['false'] if $time < $min;
-         $cursor->recover($cache);
+         return False if $time < $min;
+         $cursor->reset_cache($cache);
          return $gather;
       }
       $time++;
@@ -188,31 +194,19 @@ sub match_rept {
    return $gather;
 }
 
-sub get_rept_time {
-   my $rept = shift;
-   given ($rept) {
-      when ('?')  { return (0,  1) }  
-      when ('*')  { return (0, -1) } 
-      when ('+')  { return (1, -1) } 
-      when ('*?') { return (0, -1) }
-      when ('+?') { return (1, -1) }
-      default { error("error rept str: <$rept>!") }
-   }
-}
-
 sub match_look {
    my ($rule, $cursor) = @_;
    my ($rept, $atom, $look) = @{$rule};
    my ($min, $max) = get_rept_time($rept);
-   my ($gather, $time)   = (['true'], 0);
+   my ($gather, $time)   = (True, 0);
    while ($time != $max) {
       my $cache = $cursor->cache;
       my $match = match_rule($atom, $cursor);
       if (is_false($match)) {
-         return ['false'] if $time > $min;
-         $cursor->recover($cache);
+         return False if $time > $min;
+         $cursor->reset_cache($cache);
          $match = match_rule($look, $cursor);
-         return ['false'] if is_false($match);
+         return False if is_false($match);
          return gather_match($gather, $match);
       }
       $time++;
@@ -220,28 +214,28 @@ sub match_look {
       if ($time >= $min) {
          $cache = $cursor->cache;
          $match = match_rule($look, $cursor);
-         if (is_match($match)) {
+         if (!is_false($match)) {
             return gather_match($gather, $match)
          }
-         $cursor->recover($cache);
+         $cursor->reset_cache($cache);
       }
    }
-   return ['false'];
+   return False;
 }
 
 sub match_str {
    my ($str, $cursor) = @_;
    for my $char (split('', $str)) {
-      if ($char ne $cursor->get_char) { return ['false'] }
-      $cursor->go;
+      if ($char ne $cursor->get_char) { return False }
+      $cursor->to_next;
    }
    return $str;
 }
 
 sub match_char {
    my ($char, $cursor) = @_;
-   return ['false'] if $char ne $cursor->get_char;
-   $cursor->go;
+   return False if $char ne $cursor->get_char;
+   $cursor->to_next;
    return $char;
 }
 
@@ -250,21 +244,21 @@ sub match_chclass {
    my $char = $cursor->get_char;
    for my $atom (@{$atoms}) {
       if (match_catom($atom, $char)) {
-         $cursor->go;
+         $cursor->to_next;
          return $char;
       }
    }
-   return ['false'];
+   return False;
 }
 
 sub match_nchclass {
    my ($atoms, $cursor) = @_;
    my $char = $cursor->get_char;
-   return ['false'] if $char eq chr(0);
+   return False if $char eq chr(0);
    for my $atom (@{$atoms}) {
-      return ['false'] if match_catom($atom, $char);
+      return False if match_catom($atom, $char);
    }
-   $cursor->go;
+   $cursor->to_next;
    return $char;
 }
 
@@ -275,43 +269,41 @@ sub match_catom {
       when ("Range") { match_range($value, $char) }
       when ("Cclass") { is_match_cclass($value, $char) }
       when ("Char") { return $value eq $char }
-      default { error("unknown chclass node: <$name>!") }
    }
 }
 
 sub match_cclass {
    my ($cclass, $cursor) = @_;
    my $char = $cursor->get_char;
-   return ['false'] if $char eq chr(0);
+   return False if $char eq chr(0);
    if (is_match_cclass($cclass, $char)) {
-      $cursor->go;
+      $cursor->to_next;
       return $char;
    }
-   return ['false'];
+   return False;
 }
 
 sub is_match_cclass {
    my ($cchar, $char) = @_;
    given ($cchar) {
-      when ('a') { return is_char_alpha($char) }
-      when ('A') { return !is_char_alpha($char) }
-      when ('d') { return is_char_digit($char) }
-      when ('D') { return !is_char_digit($char) }
-      when ('h') { return is_char_hspace($char) }
-      when ('H') { return !is_char_hspace($char) }
-      when ('l') { return is_char_lower($char) }
-      when ('L') { return !is_char_lower($char) }
-      when ('s') { return is_char_space($char) }
-      when ('S') { return !is_char_space($char) }
-      when ('u') { return is_char_upper($char) }
-      when ('U') { return !is_char_upper($char) }
-      when ('v') { return is_char_vspace($char) }
-      when ('V') { return !is_char_vspace($char) }
-      when ('w') { return is_char_words($char) }
-      when ('W') { return !is_char_words($char) }
-      when ('x') { return is_char_xdigit($char) }
-      when ('X') { return !is_char_xdigit($char) }
-      default    { error("unknown cclass $cchar") }
+      when ('a') { return is_alpha($char) }
+      when ('A') { return !is_alpha($char) }
+      when ('d') { return is_digit($char) }
+      when ('D') { return !is_digit($char) }
+      when ('h') { return is_hspace($char) }
+      when ('H') { return !is_hspace($char) }
+      when ('l') { return is_lower($char) }
+      when ('L') { return !is_lower($char) }
+      when ('s') { return is_space($char) }
+      when ('S') { return !is_space($char) }
+      when ('u') { return is_upper($char) }
+      when ('U') { return !is_upper($char) }
+      when ('v') { return is_vspace($char) }
+      when ('V') { return !is_vspace($char) }
+      when ('w') { return is_words($char) }
+      when ('W') { return !is_words($char) }
+      when ('x') { return is_xdigit($char) }
+      when ('X') { return !is_xdigit($char) }
    }
 }
 
@@ -330,24 +322,21 @@ sub match_expr {
 sub match_sym {
    my ($name, $cursor) = @_;
    my $value = eval_sym($name, $cursor);
-   my $match = match_atom($value, $cursor);
-   if (is_false($match)) { return $match }
-   my $token_name = substr($name, 1);
-   my $char = first($token_name);
-   if (is_char_upper($char)) { return [$token_name, $match] }
-   if ($char eq '_') { return ['true'] }
-   return $match;
+   return match_atom($value, $cursor);
 }
 
 sub match_atom {
    my ($atom, $cursor) = @_;
    return $atom if is_bool($atom);
    my ($name, $value) = @{$atom};
-   return ['false'] if len($value) == 0;
+   return False if len($value) == 0;
    given ($name) {
-      when ('Array') { return match_lbranch($value, $cursor) }
-      when ('Str') { return match_str($value, $cursor) }
-      default { error("not implement atom $name!") }
+      when ('Array') {
+         return match_branch($value, $cursor)
+      }
+      when ('Str') {
+        return match_str($value, $cursor)
+     }
    }
 }
 
@@ -383,9 +372,6 @@ sub eval_atom {
       when ('Sym')   { return eval_sym($value, $cursor) }
       when ('Expr')  { return eval_expr($value, $cursor) }
       when ('Array') { return eval_array($value, $cursor) }
-      default {
-         $cursor->error("Could not eval atom: <$name>!");
-      }
    }
 }
 
@@ -422,13 +408,13 @@ sub eval_array {
 sub eval_push {
    my ($atoms, $cursor) = @_;
    my $sym = $atoms->[0];
-   if (is_atom_sym($sym)) {
+   if (is_sym($sym)) {
       my $eval_atoms = eval_atoms($atoms, $cursor);
       my ($array, $element) = @{$eval_atoms};
       push @{ $array->[1] }, $element;
       my $name = $sym->[1];
       $cursor->{'ns'}{$name} = $array;
-      return ['true'];
+      return True;
    }
    $cursor->error('push only accept array symbol!');
 }
@@ -437,10 +423,10 @@ sub eval_my {
    my ($atoms, $cursor) = @_;
    my $sym = $atoms->[0];
    my $value = eval_atom($atoms->[1], $cursor);
-   if (is_atom_sym($sym)) {
+   if (is_sym($sym)) {
       my $name = $sym->[1];
       $cursor->{'ns'}{$name} = $value;
-      return ['true'];
+      return True;
    }
    $cursor->error('only assign symbol!');
 }
@@ -451,7 +437,7 @@ sub eval_say {
    my $str = $eval_atoms->[0];
    if (is_atom_str($str)) {
       say $str->[1];
-      return ['true'];
+      return True;
    }
    my $type = $str->[0];
    $cursor->error("say only accept Str: <$type>");
